/*
Copyright (c) 2002-2011 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: 
*/ 

#ifndef __IMAGE_READER_H__
#define __IMAGE_READER_H__

#include <ImageConversion.h>
#include <COEMAIN.H>
#include <APGCLI.H>

class MImageReaderCallBack
	{// this callback function is used for informing the UI container
	 // that the image is now ready to be drawn
	 // (or error indicating failure to open the file)
	public:
		virtual void ImageReadDoneL(TInt aError) = 0;
	};
	
	class CImage_Reader : public CActive
	{
	public:// public constructors & destructors
		static CImage_Reader* NewL(MImageReaderCallBack& aCallBack,const TDesC& aFileName);
		static CImage_Reader* NewLC(MImageReaderCallBack& aCallBack,const TDesC& aFileName);
	    ~CImage_Reader();
		// a public function for accessing the bitmap image
	    CFbsBitmap*	Bitmap(void){ return iFrameImg;};
	protected:// active object functions
		void DoCancel();
		void RunL();
	private:// used internally to check the file's MIME type
		TInt GetFileType(const TDesC& aFileName, TDes8& aFileType);
		// private constructors
		CImage_Reader(MImageReaderCallBack& aCallBack);
	    void ConstructL(const TDesC& aFileName);
	private:// private member variables
		MImageReaderCallBack&	iCallBack;
		CImageDecoder*			iImageDecoder;
		CFbsBitmap*				iFrameImg;
		RFs						iFsSession;
	};


#endif // __IMAGE_READER_H__

